@@ -21,6 +21,7 @@ use {
         BitString, Captured, ConstOid, Integer, Mode, OctetString, Oid, Tag,
     },
     std::{
+        fmt::{Debug, Formatter},
         io::Write,
         ops::{Deref, DerefMut},
     },
@@ -201,11 +202,9 @@ impl SignedData {
                     self.version.encode(),
                     self.digest_algorithms.encode_ref(),
                     self.content_info.encode_ref(),
-                    if let Some(certs) = self.certificates.as_ref() {
-                        Some(certs.encode_ref_as(Tag::CTX_0))
-                    } else {
-                        None
-                    },
+                    self.certificates
+                        .as_ref()
+                        .map(|certs| certs.encode_ref_as(Tag::CTX_0)),
                     // TODO crls.
                     self.signer_infos.encode_ref(),
                 )),
@@ -303,12 +302,29 @@ impl SignerInfos {
 ///   eContentType ContentType,
 ///   eContent [0] EXPLICIT OCTET STRING OPTIONAL }
 /// ```
-#[derive(Clone, Debug, Eq, PartialEq)]
+#[derive(Clone, Eq, PartialEq)]
 pub struct EncapsulatedContentInfo {
     pub content_type: ContentType,
     pub content: Option<OctetString>,
 }
 
+impl Debug for EncapsulatedContentInfo {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        let mut s = f.debug_struct("EncapsulatedContentInfo");
+        s.field("content_type", &format_args!("{}", self.content_type));
+        s.field(
+            "content",
+            &format_args!(
+                "{:?}",
+                self.content
+                    .as_ref()
+                    .map(|x| hex::encode(x.clone().to_bytes().as_ref()))
+            ),
+        );
+        s.finish()
+    }
+}
+
 impl EncapsulatedContentInfo {
     pub fn take_from<S: Source>(cons: &mut Constructed<S>) -> Result<Self, S::Err> {
         cons.take_sequence(|cons| {
@@ -326,11 +342,9 @@ impl EncapsulatedContentInfo {
     pub fn encode_ref(&self) -> impl Values + '_ {
         encode::sequence((
             self.content_type.encode_ref(),
-            if let Some(content) = self.content.as_ref() {
-                Some(encode::sequence_as(Tag::CTX_0, content.encode_ref()))
-            } else {
-                None
-            },
+            self.content
+                .as_ref()
+                .map(|content| encode::sequence_as(Tag::CTX_0, content.encode_ref())),
         ))
     }
 }
@@ -347,7 +361,7 @@ impl EncapsulatedContentInfo {
 ///   signature SignatureValue,
 ///   unsignedAttrs [1] IMPLICIT UnsignedAttributes OPTIONAL }
 /// ```
-#[derive(Clone, Debug, Eq, PartialEq)]
+#[derive(Clone, Eq, PartialEq)]
 pub struct SignerInfo {
     pub version: CmsVersion,
     pub sid: SignerIdentifier,
@@ -421,19 +435,15 @@ impl SignerInfo {
             u8::from(self.version).encode(),
             &self.sid,
             &self.digest_algorithm,
-            if let Some(attrs) = self.signed_attributes.as_ref() {
-                // Always write signed attributes with DER encoding per RFC 5652.
-                Some(SignedAttributesDer::new(attrs.clone(), Some(Tag::CTX_0)))
-            } else {
-                None
-            },
+            // Always write signed attributes with DER encoding per RFC 5652.
+            self.signed_attributes
+                .as_ref()
+                .map(|attrs| SignedAttributesDer::new(attrs.clone(), Some(Tag::CTX_0))),
             &self.signature_algorithm,
             self.signature.encode_ref(),
-            if let Some(attrs) = self.unsigned_attributes.as_ref() {
-                Some(attrs.encode_ref_as(Tag::CTX_1))
-            } else {
-                None
-            },
+            self.unsigned_attributes
+                .as_ref()
+                .map(|attrs| attrs.encode_ref_as(Tag::CTX_1)),
         ))
     }
 
@@ -507,6 +517,7 @@ impl SignerInfo {
                 // No existing copy present. Serialize from raw data structures.
                 // But we obtain a sorted instance of those attributes first, because
                 // bcder doesn't appear to follow DER encoding rules for sets.
+                let signed_attributes = signed_attributes.as_sorted()?;
                 let mut der = Vec::new();
                 // The mode argument here is actually ignored.
                 signed_attributes.write_encoded(Mode::Der, &mut der)?;
@@ -519,6 +530,34 @@ impl SignerInfo {
     }
 }
 
+impl Debug for SignerInfo {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        let mut s = f.debug_struct("SignerInfo");
+
+        s.field("version", &self.version);
+        s.field("sid", &self.sid);
+        s.field("digest_algorithm", &self.digest_algorithm);
+        s.field("signed_attributes", &self.signed_attributes);
+        s.field("signature_algorithm", &self.signature_algorithm);
+        s.field(
+            "signature",
+            &format_args!(
+                "{}",
+                hex::encode(self.signature.clone().into_bytes().as_ref())
+            ),
+        );
+        s.field("unsigned_attributes", &self.unsigned_attributes);
+        s.field(
+            "signed_attributes_data",
+            &format_args!(
+                "{:?}",
+                self.signed_attributes_data.as_ref().map(hex::encode)
+            ),
+        );
+        s.finish()
+    }
+}
+
 impl Values for SignerInfo {
     fn encoded_len(&self, mode: Mode) -> usize {
         self.encode_ref().encoded_len(mode)
